unit Unitcomp;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, 
  Unitglob,
  Unittime,
  Unitconf,
  Unittool,
  Unithshk, AdPort, Async32;

type
  TFormComp = class(TForm)
    GroupBoxComp: TGroupBox;
    RadioButtonCompCom1: TRadioButton;
    RadioButtonCompCom2: TRadioButton;
    RadioButtonCompCom3: TRadioButton;
    RadioButtonCompCom4: TRadioButton;
    Image1: TImage;
    RadioButtonCompComAuto: TRadioButton;
    GroupBoxComSpeed: TGroupBox;
    Image2: TImage;
    RadioButtonCompSpeed38k4: TRadioButton;
    RadioButtonCompSpeed57k6: TRadioButton;
    RadioButtonCompSpeed115k2: TRadioButton;
    BitBtnCompOK: TBitBtn;
    BitBtnCompAbort: TBitBtn;
    AsyncCom: TComm;
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormComp: TFormComp;

Procedure SetConfigurationDefaults;
Procedure OpenCommunication;
Procedure CloseCommunication;
Procedure SendChar(C : Char);
Procedure SendString(S : String);
Procedure SendBlock(B : TBinaryBuffer; BLen : Integer);
Function  ReadByte : Byte;
Procedure ReadBlock(Var B : TBinaryBuffer; BLen : Integer);
Procedure ReadRecord(Var R : TBIRec; RLen : Integer);
Procedure RTSDTROn;
Procedure RTSOn;
Procedure RTSOff;
Procedure RTSPulse;
Procedure DTROff;
Procedure SetFrameCTSOn;
Procedure SetFrameCTSOff;
Procedure WFlushOutSIOBuffer;
Procedure FlushInSIOBuffer;
Procedure DeviceWarmstart;
Procedure DeviceColdstart;
Procedure SendCommunicationHead;
Procedure SendAndSetSpeed;
Procedure SetNewComPort;
Procedure SetComNrAutomatic;
Procedure AskAndRead1s( OutString : String );
Procedure AskAndRead5s( OutString : String );
Procedure PutComString( OutStr : String );
Procedure PutComByte( B : Byte );
Procedure PutComArray;
Procedure GetComArray;
Function GetOutFree : LongInt;
Function GetInOccup : LongInt;
Procedure WaitAndRead1s;
Procedure WaitAndRead16s;
Procedure WaitAndRead120s;
Procedure WaitAndRead120sWB;
Procedure ReadInfos;
Function  CrHere : Boolean;
Procedure Wait200msRefr;
Procedure Wait500msRefr;
Procedure Wait1000msRefr;
Procedure EstablishConnection;

implementation

{$R *.DFM}



Procedure SetConfigurationDefaults;
Begin
  ComNr:=1;
  ComSpeed:=57600
End;


Procedure OpenCommunication;
Begin
  ComNrOkay:=True;
  Try
    With FormComp.AsyncCom Do
      Begin
        DeviceName:='COM'+IntToStr(ComNr);
        Open;
        BaudRate:=cbr2400;
        Parity:=paNone;
        DataBits:=da8;
        StopBits:=sb20;
        SetRTSState(True);
        SetDTRState(True);
        Wait1000msRefr
      End
  Except
    FormComp.AsyncCom.Close;
    ComNrOkay:=False
  End
End;



Procedure SetBDRate(BR : LongInt);
Begin
  With FormComp.AsyncCom Do
    Begin
      If BR=38400 Then
        BaudRate:=cbr38400;
      If BR=57600 Then
        BaudRate:=cbr57600;
      If BR=115200 Then
        BaudRate:=cbr115200
    End
End;

Procedure CloseCommunication;
Begin
  Wait1000msRefr;
  With FormComp.AsyncCom Do
    Begin
      SetRTSState(False);
      SetDTRState(False);
      Close
    End
End;



Procedure SendChar(C : Char);
Begin
  FormComp.AsyncCom.Write(C,1)
End;



Procedure SendString(S : String);
Var I : Integer;
Begin
  For I:=1 To Length(S) Do
    Begin
      If S[I]=Cr Then
        Begin
          RestartTimer50ms;
          Repeat
            Application.ProcessMessages
          Until Timeout50ms
        End;
      SendChar(S[I])
    End
End;



Procedure SendBlock(B : TBinaryBuffer; BLen : Integer);
Begin
  FormComp.AsyncCom.Write(B,BLen)
End;



Function ReadByte : Byte;
Var B : Byte;
Begin
  FormComp.AsyncCom.Read(B,1);
  ReadByte:=B
End;



Procedure ReadBlock(Var B : TBinaryBuffer; BLen : Integer);
Begin
  FormComp.AsyncCom.Read(B,BLen)
End;


Procedure ReadRecord(Var R : TBIRec; RLen : Integer);
Begin
  FormComp.AsyncCom.Read(R,RLen)
End;



Procedure RTSDTROn;
Begin
  With FormComp.AsyncCom Do
    Begin
      SetRTSState(True);
      SetDTRState(True)
    End
End;



Procedure RTSOn;
Begin
  FormComp.AsyncCom.SetRTSState(True)
End;

Procedure RTSOff;
Begin
  FormComp.AsyncCom.SetRTSState(False)
End;

Procedure RTSPulse;
Begin
  RTSOff;
  Wait200msRefr;
  RTSOn
End;

Procedure DTROff;
Begin
  FormComp.AsyncCom.SetDTRState(False)
End;


Procedure SetFrameCTSOn;
Begin
  FormComp.AsyncCom.FlowControl:=fcCTS
End;

Procedure SetFrameCTSOff;
Begin
  Repeat
    Application.ProcessMessages
  Until (FormComp.AsyncCom.OutQueCount=0);
  FormComp.AsyncCom.FlowControl:=fcNone
End;


Procedure WFlushOutSIOBuffer;
Begin
  Repeat
  Until (GetOutFree=FormComp.AsyncCom.WriteBufSize)
End;



Procedure FlushInSIOBuffer;
Begin
  FormComp.AsyncCom.PurgeIn;
  Application.ProcessMessages
End;



Procedure DeviceWarmstart;
Begin
  SendChar('E')
End;



Procedure DeviceColdstart;
Begin
  SendChar('!')
End;



Procedure SendCommunicationHead;
Begin
  Character:='-';
  RestartTimer500ms;
  SendString(CommunicationHead);
  Repeat
    Application.ProcessMessages;
    While FormComp.AsyncCom.InQueCount<>0 Do
      Begin
        Character:=Chr(ReadByte);
        If (Character=Device_IDLE) Then
          DevicePresent:=True
      End
  Until (Timeout500ms Or DevicePresent)
End;



Procedure SendAndSetSpeed;
Begin
  RestartTimer50ms;
  Repeat
    Application.ProcessMessages
  Until Timeout50ms;
  If ComSpeed=38400 Then
    SendChar('6');
  If ComSpeed=57600 Then
    SendChar('7');
  If ComSpeed=115200 Then
    SendChar('8');
  RestartTimer200ms;
  Repeat
    Application.ProcessMessages
  Until Timeout200ms;
  SetBDRate(ComSpeed);
  RestartTimer500ms;
  Repeat
    Application.ProcessMessages
  Until Timeout500ms
End;


Procedure SetNewComPort;
Begin
  FormComp.RadioButtonCompComAuto.Enabled:=True;
  FormComp.BitBtnCompAbort.Enabled:=True;
  FormComp.BitBtnCompOK.Enabled:=True;
  If Not(FormComp.RadioButtonCompComAuto.Checked) Then
    Case ComNr Of
      1 : FormComp.RadioButtonCompCom1.Checked:=True;
      2 : FormComp.RadioButtonCompCom2.Checked:=True;
      3 : FormComp.RadioButtonCompCom3.Checked:=True;
      4 : FormComp.RadioButtonCompCom4.Checked:=True
    End;
  If ComSpeed=38400 Then
     FormComp.RadioButtonCompSpeed38k4.Checked:=True;
  If ComSpeed=57600 Then
     FormComp.RadioButtonCompSpeed57k6.Checked:=True;
  If ComSpeed=115200 Then
     FormComp.RadioButtonCompSpeed115k2.Checked:=True;
  FormComp.ShowModal;
  If (FormComp.ModalResult=mrCancel) Or
     (FormComp.ModalResult=mrAbort) Then
    Done:=True;
  If FormComp.ModalResult=mrOK Then
    Begin
      If FormComp.RadioButtonCompComAuto.Checked Then
        Begin
          FormComp.Show;
          FormComp.BitBtnCompAbort.Enabled:=False;
          FormComp.BitBtnCompOK.Enabled:=False;
          Application.ProcessMessages;
          SetComNrAutomatic;
          FormComp.BitBtnCompAbort.Enabled:=True;
          FormComp.BitBtnCompOK.Enabled:=True;
          FormComp.Hide
        End;
      If FormComp.RadioButtonCompCom1.Checked Then
        ComNr:=1;
      If FormComp.RadioButtonCompCom2.Checked Then
        ComNr:=2;
      If FormComp.RadioButtonCompCom3.Checked Then
        ComNr:=3;
      If FormComp.RadioButtonCompCom4.Checked Then
        ComNr:=4;
      If FormComp.RadioButtonCompSpeed38k4.Checked Then
        ComSpeed:=38400;
      If FormComp.RadioButtonCompSpeed57k6.Checked Then
        ComSpeed:=57600;
      If FormComp.RadioButtonCompSpeed115k2.Checked Then
        ComSpeed:=115200
    End
End;

Procedure SetComNrAutomatic;
Var LocComNr : Integer;
Begin
  For LocComNr:=1 To 4 Do
    Begin
      Application.ProcessMessages;
      ComNr:=LocComNr;
      OpenCommunication;
      If ComNrOkay Then
        Begin
          CloseCommunication
        End
      Else
        Case LocComNr Of
          1 : FormComp.RadioButtonCompCom1.Enabled:=False;
          2 : FormComp.RadioButtonCompCom2.Enabled:=False;
          3 : FormComp.RadioButtonCompCom3.Enabled:=False;
          4 : FormComp.RadioButtonCompCom4.Enabled:=False
        End;
      Application.ProcessMessages
    End;
  For LocComNr:=1 To 4 Do
    Begin
      Case LocComNr Of
        1 : FormComp.RadioButtonCompCom1.Checked:=True;
        2 : FormComp.RadioButtonCompCom2.Checked:=True;
        3 : FormComp.RadioButtonCompCom3.Checked:=True;
        4 : FormComp.RadioButtonCompCom4.Checked:=True
      End;
      Application.ProcessMessages;
      ComNr:=LocComNr;
      OpenCommunication;
      If ComNrOkay Then
        Begin
          SendCommunicationHead;
          If DevicePresent Then
            Begin
              SendChar('E');
              ComNr:=LocComNr;
              FormComp.RadioButtonCompComAuto.Enabled:=False;
              Exit
            End;
          CloseCommunication
        End
    End;
  FormComp.RadioButtonCompComAuto.Checked:=True
End;


Procedure AskAndRead1s( OutString : String );
Begin
  DeviceFailure:=False;
  InputString:='';
  Character:='-';
  RestartTimer1000ms;
  PutComString(OutString);
  Repeat
    If FormComp.AsyncCom.InQueCount<>0 Then
      Begin
        Character:=Chr(ReadByte);
        InputString:=InputString+Character
      End;
    Application.ProcessMessages
  Until Timeout1000ms Or (Character=Cr);
  If Timeout1000ms Then
    DeviceFailure:=True
End;


Procedure AskAndRead5s( OutString : String );
Begin
  DeviceFailure:=False; 
  InputString:='';
  Character:='-';
  RestartTimer5000ms;
  PutComString(OutString);
  Repeat
    If FormComp.AsyncCom.InQueCount<>0 Then
      Begin
        Character:=Chr(ReadByte);
        InputString:=InputString+Character
      End;
    Application.ProcessMessages
  Until Timeout5000ms Or (Character=Cr);
  If Timeout5000ms Then
    DeviceFailure:=True
End;


Procedure PutComString( OutStr : String );
Begin
  With FormComp.AsyncCom Do
    Begin
      PurgeIn;
      PurgeOut
    End;
  Application.ProcessMessages;
  SendString(OutStr)
End;



Procedure PutComByte( B : Byte );
Begin
  Repeat
    Application.ProcessMessages
  Until (FormComp.AsyncCom.OutQueCount < FormComp.AsyncCom.WriteBufSize);
  SendChar(Chr(B))
End;


Procedure PutComArray;
Begin
  SendBlock(BinaryBuffer,OutRecordLen)
End;


Procedure GetComArray;
Begin
  Application.ProcessMessages;
  InRecordLen:=FormComp.AsyncCom.InQueCount;
  If InRecordLen>0 Then
    ReadBlock(BinaryBuffer,InRecordLen)
End;


Function GetOutFree : LongInt;
Begin
  Application.ProcessMessages;
  GetOutFree:=FormComp.AsyncCom.WriteBufSize-FormComp.AsyncCom.OutQueCount
End;


Function GetInOccup : LongInt;
Begin
  Application.ProcessMessages;
  GetInOccup:=FormComp.AsyncCom.InQueCount
End;


Procedure WaitAndRead1s;
Begin
  DeviceFailure:=False;
  InputString:='';
  Character:='-';
  RestartTimer1000ms;
  Repeat
    If FormComp.AsyncCom.InQueCount<>0 Then
      Begin
        Character:=Chr(ReadByte);
        InputString:=InputString+Character
      End;
    Application.ProcessMessages
  Until Timeout1000ms Or (Character=Cr);
  If Timeout1000ms Then
    DeviceFailure:=True
End;


Procedure WaitAndRead16s;
Begin
  DeviceFailure:=False;
  InputString:='';
  Character:='-';
  RestartTimer16000ms;
  Repeat
    If FormComp.AsyncCom.InQueCount<>0 Then
      Begin
        Character:=Chr(ReadByte);
        InputString:=InputString+Character
      End;
    Application.ProcessMessages
  Until Timeout16000ms Or (Character=Cr);
  If Timeout16000ms Then
    DeviceFailure:=True
End;


Procedure WaitAndRead120s;
Begin
  DeviceFailure:=False;
  InputString:='';
  Character:='-';
  DoubleTimer:=False;
  RestartTimer60000ms;
  Repeat
    If FormComp.AsyncCom.InQueCount<>0 Then
      Begin
        Character:=Chr(ReadByte);
        InputString:=InputString+Character
      End;
    Application.ProcessMessages;
    If Timeout60000ms And Not(DoubleTimer) Then
      Begin
        DoubleTimer:=True;
        RestartTimer60000ms
      End
  Until (Timeout60000ms And DoubleTimer) Or (Character=Cr);
  If (Timeout60000ms And DoubleTimer) Then
    DeviceFailure:=True
End;


Procedure WaitAndRead120sWB;
Begin
  DeviceFailure:=False;
  InputString:='';
  Character:='-';
  DoubleTimer:=False;
  RestartTimer60000ms;
  Repeat
    If FormComp.AsyncCom.InQueCount<>0 Then
      Begin
        Character:=Chr(ReadByte);
        InputString:=InputString+Character
      End;
    Application.ProcessMessages;
    If Timeout60000ms And Not(DoubleTimer) Then
      Begin
        DoubleTimer:=True;
        RestartTimer60000ms
      End;
    If BreakTimer Then
      PutComByte(Ord(Cr))
  Until (Timeout60000ms And DoubleTimer) Or (Character=Cr);
  If (Timeout60000ms And DoubleTimer) Then
    DeviceFailure:=True
End;



Procedure ReadInfos;
Begin
  AskAndRead1s('?');
  If DeviceFailure Then
    Exit;
  RemoveChar(InputString,Cr);
  If Length(InputString)<>4 Then
    Begin
      DeviceFailure:=True;
      Exit
    End;
  ActNumVersion:=StrToInt(Copy(InputString,1,3));
  ActLangVersion:=InputString[4];
  ActStrVersion:=InputString[1]+'.'+Copy(InputString,2,3);

  (* Device recognition *)

  (* InputString = 3.3x - 3.4x ............. old MemoFax, only RTS, no FLASH
                   3.5x upto   ............. new MemoFax, RTS/DTR, FLASH AMD29010
                   4.3x        ............. old SaferFax, only RTS, no FLASH
                   5.3x        ............. old SaferFax, only RTS, FLASH ATMEL29010
                   5.5x upto   ............. new SaferFax, RTS/DTR, FLASH AMD29010 *)

  DeviceType:=MemoFax;
  DeviceStr:='Unbekannte Hardware';
  FlashOption:=FlashOptionNone;
  RTSDTR:=False;

  If (ActNumVersion>=330) And
     (ActNumVersion<=349) Then
    Begin
      DeviceType:=MemoFax;
      DeviceStr:='MemoFax';
      FlashOption:=FlashOptionNone;
      RTSDTR:=False
    End;
  If (ActNumVersion>=350) And
     (ActNumVersion<=399) Then
    Begin
      DeviceType:=MemoFax;
      DeviceStr:='MemoFax';
      FlashOption:=FlashOptionAMD;
      RTSDTR:=True
    End;
  If (ActNumVersion>=430) And
     (ActNumVersion<=499) Then
    Begin
      DeviceType:=SaferFax;
      DeviceStr:='SaferFax';
      FlashOption:=FlashOptionNone;
      RTSDTR:=False
    End;
  If (ActNumVersion>=530) And
     (ActNumVersion<=549) Then
    Begin
      DeviceType:=SaferFax;
      DeviceStr:='SaferFax';
      FlashOption:=FlashOptionATMEL;
      RTSDTR:=False
    End;
  If (ActNumVersion>=550) And
     (ActNumVersion<=599) Then
    Begin
      DeviceType:=SaferFax;
      DeviceStr:='SaferFax';
      FlashOption:=FlashOptionAMD;
      RTSDTR:=True
    End;

  (* Memory chips recognition *)

  AskAndRead1s('M');
  If DeviceFailure Then
    Exit;
  RemoveChar(InputString,Cr);
  If Length(InputString)<>2 Then
    Begin
      DeviceFailure:=True;
      Exit
    End;
  MemBase:=StrToInt(InputString[1]);
  MemExtension:=StrToInt(InputString[2]);
  Case MemBase Of
    0    : Begin
             MemString:='256kB'
           End;
    1    : Begin
             MemString:='1MB'
           End;
    2    : Begin
             MemString:='4MB'
           End
    Else   Begin
             MemString:='???'
           End
  End;
  Case MemExtension Of
    0    : Begin
             MemString:=MemString+' + SIMM ---'
           End;
    1    : Begin
             MemString:=MemString+' + SIMM 256kB'
           End;
    2    : Begin
             MemString:=MemString+' + SIMM 1MB'
           End;
    3    : Begin
             MemString:=MemString+' + SIMM 4MB'
           End
    Else   Begin
             MemString:=MemString+' + SIMM ???'
           End
  End;

  AskAndRead1s('N');
  If DeviceFailure Then
    Exit;
  RemoveChar(InputString,Cr);
  If (Length(InputString)<>2) Then
    Begin
      DeviceFailure:=True;
      Exit
    End;
  NFaxes:=StrToInt('$'+InputString[1]+InputString[2])
End;


Function CrHere : Boolean;
Begin
  CrHere:=False;
  While (FormComp.AsyncCom.InQueCount>0) Do
    Begin
      Application.ProcessMessages;
      If ReadByte=Ord(Cr) Then
        CrHere:=True
    End
End;


Procedure Wait200msRefr;
Begin
  RestartTimer200ms;
  Repeat
    Application.ProcessMessages
  Until Timeout200ms
End;

Procedure Wait500msRefr;
Begin
  RestartTimer500ms;
  Repeat
    Application.ProcessMessages
  Until Timeout500ms
End;

Procedure Wait1000msRefr;
Begin
  RestartTimer1000ms;
  Repeat
    Application.ProcessMessages
  Until Timeout1000ms
End;


Procedure EstablishConnection;
Begin
  If Not(DeviceConnected) Then
    Begin
      FormHandshake.Show;
      SendCommunicationHead;
      If Not(DevicePresent) Then
        Begin
          FormHandshake.Hide;
          Exit
        End;
      SendAndSetSpeed;
      If DeviceFailure Then
        Exit;
      FormHandshake.Hide
    End;
  DeviceConnected:=True;
  ReadInfos
End;

end.
